 /*
  * Author Alex
  * about September 2018
  *  DEF PUMA Pop Up Menu Add
  * usage: 
  * create a div D to be popped up (may use create_popup_div)
  * target items (those which must show the menu on right click) should have some specific class C
  * call init_popup in onload, passing created D and C (if C empty - popup will show on any item)
  *
  * CSS: (now in table_tree/css/style.css)
	.ttc_popup_div{
		border: 1px solid grey;
		background-color: GhostWhite;
		box-shadow: 4px 4px 2px grey;
	}
	.ttc_popup_div div{
		padding: 5;
		color: #334E67;
	}
  */
/* problem of hiding menu (when using a one handler for each menu): 
	on mouse click need close previously opened menus; suppose have several menus; menu handler was called and has opened the menu;
	then is called the handler of next menu - and it closes the prev menu; to avoid that - need a single menu handler; thus need keep data in a var
*/
var popup_menus = []; // arr of obj: class,menu,hanlder

// a shorthand that may be used to create a div holding menu items
function create_popup_div(div_id=""){
	var div = $("<div id='" + div_id + "' class='ttc_popup_div hidden' style='position: absolute; z-index: 1000; cursor: pointer;'>");
	$(document.body).append(div);
	return div;
}
// 4.02.20 added parent_id ; there was problem that popup was assigned to .mc_field_capt used by all tables; this allows to specify parent container id
// 17.06.22 with CUS may use class as parent_id (with point, eg ".bla" )
function init_popup(popup_div, target_class='', callback=null, onleftclick=false, parent_id=""){	 // new 15.11.19 callback is used to do sms before popping up; ie enable only necessary items; param = target,popup_div
	popup_menus = popup_menus.filter( item => item.tc!=target_class ); // remove prev version (if any)
	popup_menus.push( {"tc" : target_class, "menu" : popup_div, "cb" : callback, "left" : onleftclick, "parent_id" : parent_id} );
}

$(document).ready( function(){
	// close menu on escape click
	window.addEventListener('keyup', function(e) { // keypress may not work (works only for symbol keys)
		if(e.key==="Escape")
			$(".ttc_popup_div").addClass("hidden"); // hide all menues on any click
	}, false);

	// detect right click and call click action
	if (document.addEventListener) { // IE >= 9; other browsers
		document.addEventListener('contextmenu', check_all_menus, false);
	} else { // IE < 9
		document.attachEvent('oncontextmenu', function() {
			window.event.returnValue = !check_all_menus(window.event); // donno, guess returnValue = !processed
		});  
	} 

	// close popup on any mouse click (within or out of menu)
	document.addEventListener('click', function(e) {check_all_menus(e, true);}, false);
}); // end of ready function


// variable which holds clicked element; is used in menu item click handler
var popup_target;
function set_popup_target(target){  // PM_EMB used by client to use another target (not the clicked elem); used when user calls menu by clicking special button (used in MOMO)
	popup_target = target;
}


// single click handler for all menus
function check_all_menus(e, leftclick=false){
	$(".ttc_popup_div").addClass("hidden"); // hide all menues on any click
	for(i=0;i<popup_menus.length;i++){
		var menu = popup_menus[i];
		if( rightClickAction(e, menu.menu, menu.tc, menu.cb, menu.left, leftclick, menu.parent_id) )
			return; 
	}
}
var popup_x;
var popup_y;
function get_popup_point(){return {x: popup_x, y: popup_y};}
// action on right click; must find out if popup necessary; if yes - show it; return res = popup was shown
// called for each registered menus; checks if this menu is applicable to the event's target and is within defined parent
function rightClickAction(e, popup_div, target_class, callback, onleftclick, leftclick, parent_id){
	// check if clicked elem has neccessary class (if some of registered menus refer this class)
	var target = $(e.target);
	if( target_class!='' && !target.closest("." + target_class).length )
		return false;
	if(parent_id && !parent_id.startsWith("#") && !parent_id.startsWith(".")) parent_id = "#" + parent_id; // CUS
	if( parent_id && !target.closest(parent_id).length )
		return false;

	// if it was left click - show menu only if it was requested (on init)
	if( leftclick && !onleftclick )
		return false;
	// show
	popup_x = e.offsetX;	popup_y = e.offsetY;
	popup_div.css("left", e.clientX);
	popup_div.css("top", e.clientY);
	popup_div.removeClass("hidden");
	popup_target = e.target;
	if(callback)
		callback(e.target, popup_div);
	e.preventDefault();
	return true;
}

// user may use this shorthand to add a menu item; instead may create html manually and add a handler like this:
// $("#tt_close_all").click(function(){  close_dir_recursive( get_cont( $(popup_target) ) ); }); // must use the variable from this file 'popup_target' to get the target
// popup_div - jquery elem
// click_handler - func which takes single param - jquery elem = clicked target
function add_popup_item(popup_div, caption, click_handler){
	var item = $("<div class='ttc_pu_item'>" + caption + "</div>");
	item.click( function(e){ click_handler( $(popup_target), e ); } );
	popup_div.append(item);
	return item;
}
// to add item = checkbox
function add_popup_checkbox(popup_div, name, value, state, click_handler){
	var div = $("<div></div>");
	div.html(name);
	var checked = state? "checked" : "";
	var cb = $("<input style='margin-right: 5px;' type=checkbox " + checked + " value='" + value + "'>");
	cb.change( function(){
		var cb = $(this);
		var state = cb.prop("checked");
		var val = cb.val();
		click_handler( $(popup_target), val, state );
	} );
	popup_div.append(div);
	div.prepend(cb);
	return div;
}
function add_menu_separator(popup_div){
	popup_div.append("<div style='border: 1px solid grey; width: 100%; height: 1px; padding: 0px;'></div>");
}

