	<?php
		function create_files_zip($start_dir, $files=array(), $zip_name=""){ // NOTE: pass where possible zip_name to avoid another copy (with random name?)
			if(!$zip_name)
				$zip_name = temp_path.generate_random_string().".zip"; // now this file is temp, hope no collisions NOTE: some clients expect zip to be exactly in temp_path (download.php) to control if zip was created (or original was returned)
			log\debug("ZIP name: $zip_name");
			$zip = new ZipArchive; // apt-get install php5.6-zip
			if(! $zip->open($zip_name, ZipArchive::CREATE | ZipArchive::OVERWRITE))
				throw new Exception("create zip fail"); // NOTE: this may happen if file is busy (possible in ic3)
			set_time_limit(0);
			foreach($files as $file){
				$path = $start_dir.$file;
				$zpath = $file;
				log\debug("added $path $zpath");
				if( is_file($path) )
					$zip->addFile($path, $zpath);
				else
					log\error("file not found: '$path'");
			}
			if(!$zip->close())
				throw new Exception("failed to write zip '$path'");
			return $zip_name;
		}
		function download_apps($apps, $orders, $camp_id, $role){
			ob_start();
			$files = array(); // $apps is what is passed in POST and found in db; $files - what is actually present in the dir
			$path = get_source_path()."campaigns/$camp_id/lbs/";

			if(sizeof($orders))
				$apps = array_unique(array_merge($apps, 
					db\select_field_to_array("app_lbs", "id", "app_state in ('uploaded','approved','packed','complete') and ".db\id_in($orders, "", "order_id") ) ));

			if(!sizeof($apps))
				throw new user_exception("no lbs to download");

			foreach($apps as $app_id){
				if(!is_dir($path.$app_id)) { log\debug("dir not found: '$path $ap_id'"); continue; }
				$af = scandir($path.$app_id);  
				$app_files = array(); foreach($af as $f) $app_files[]="$app_id/$f";
				$files = array_merge($files, $app_files );
				// instead print_Lbs is called from send_pack // if($role=="PRODUCTION") print_lbs($app_id); // NOTE: if download from web-int - print_lbs is requested by js to contr, which is better; although this unit may also be considered a contr of dl req
			}
			$files = array_filter($files, function($f){ return in_array(extract_ext($f), array("pdf","indd","idml")); }); // TODO: now the dir contains all stuff including scales; maybe use a dedicated dir for triples
			log\debug($files);
			if(sizeof($apps)) $shop_name = db\select_field("v_lbs", "shop_name", "id=$app_id"); else return;  // 18.08.22 added shop name to zip file name
			$zip = create_files_zip($path, $files); // uses source_path from path.php
			ob_clean();
			try{
				download_file_path( $zip, $shop_name."_$camp_id.zip" );
			}catch(Exception $e){ 
				unlink($zip);
				throw $e; 
			}
			unlink($zip);
		}
		function download_lbs_file($lbs_id, $type){
			ob_start();
			if($type=="mini")
				$path = get_source_path().db\select_field("app_lbs", "concat(path,name_mini)", "id=$lbs_id");
			else{
				$order_id = db\select_field("app_lbs", "order_id", "id=$lbs_id");
				$client_code = db\select_field("v_orders", "client_code", "id=$order_id");
				$folder = $type=="triple"? "orders" : "hires";
				$path = get_source_path().$client_code."/$folder/$order_id/$lbs_id/".db\select_field("app_lbs", "name_triple", "id=$lbs_id");
			}
			ob_clean();
			download_file_path( $path, "HR_".basename($path) );
			log\info("download ok '$path'");
		}
		// download triples or hires
		function download_order_zip($order_id, $type){ // type: pdf, hires
			ob_start();
			$client_code = db\select_field("v_orders", "client_code", "id=$order_id");
			$opr = $type=="hires_all"? "hires" : "orders";
			$order_path = get_source_path().$client_code."/$opr/$order_id/"; 
			log\debug($order_path);
			$fname = $type=="hires_all"? "hr_name" : "name_triple";
			$fpath = $type=="hires_all"? "hr_name" : "concat(id,'/',name_triple) name";
			$files = db\select_field_to_array("app_lbs", $fpath, "order_id = $order_id and $fname<>'' and $fname is not null");
			$zip = create_files_zip($order_path, $files);
			ob_clean();
			try{
				$zip_name = $type.'_'.$order_id;
				download_file_path( $zip, $zip_name );
			}catch(Exception $e){ 
				unlink($zip);
				throw $e; 
			}
			unlink($zip);
		}
		function download_order_minies($order_id){ // type: pdf, hires
			ob_start();
			$files = db\select_field_to_array("app_lbs", "concat(path, name_mini) name", "order_id=$order_id");
			$order_path = get_source_path(); 
			log\debug($order_path);
			$zip = create_files_zip($order_path, $files);
			ob_clean();
			try{
				$zip_name = $order_id;
				download_file_path( $zip, $zip_name );
			}catch(Exception $e){ 
				unlink($zip);
				throw $e; 
			}
			unlink($zip);
		}
		function find_triple_files($path){
			log\debug("path: ", $path);
			$files = scandir($path);
			log\debug("files: ", $files);
			// TODO: not accurate method to filter out scales
			// return array_filter($files, function($f){ return !starts_with($f,".") && !preg_match("/^.{8}(_\d\d)?\.jpg$/", $f); });    // this ver meddles with indexes; then json_encode treats it like object (not array)
			$res = []; foreach($files as $f) if( !starts_with($f,".") && !preg_match("/^.{8}(_\d\d)?\.jpg$/", $f)  && $f!="crop.jpg" && $f!="preview.jpg" ) $res[]=$f;
			log\debug("triple files: ", $res);
			return $res;
		}
		// DEF DTP download triple problem: found out that instead of download_triple is used DTS which uses name_triple which is not set and is to dl only single file... had to uncomment this DTX
		function download_triple($lbs_id){
			ob_start();
			$lbs = db\select_record("v_order_lbs", "*", "id=$lbs_id");
			$path = source_path.get_lbs_path($lbs["id"], $lbs["order_id"], $lbs["client_code"]);
			$files = find_triple_files($path);
			log\debug("ZP $path");
			$zip = create_files_zip($path, $files); // uses source_path from path.php
			$app  = db\select_record("v_lbs", "shop_name, name", "id=$lbs_id");  // 18.08.22 added shop name to zip file name
			ob_clean();
			try{
				download_file_path( $zip, $app["shop_name"]."_".$app["name"]."_$lbs_id.zip" );
			}catch(Exception $e){ 
				unlink($zip);
				throw $e; 
			}
			unlink($zip);
		}
		function download_crop($lbs_id){
			ob_start();
			$lbs = db\select_record("v_order_lbs", "*", "id=$lbs_id");
			$path = source_path.get_lbs_path($lbs["id"], $lbs["order_id"], $lbs["client_code"])."/crop.jpg";
			if(!file_exists($path)) // if crop not exists - use mini
				$path = get_source_path().db\select_field("app_lbs", "concat(path,name_mini)", "id=$lbs_id");
			ob_clean();
			try{
				download_file_path( $path, $lbs["name"] );
			}catch(Exception $e){ 
				//unlink($zip);
				throw $e; 
			}
			//unlink($zip);
		}
		/*
		// it was used in fabri version, when there are 3 files
		function download_pdf($lbs_id){
			ob_start();
			$path = source_path.get_lbs_path($lbs_id); // TODO: check
			$files = scandir($path);
			foreach($files as $file) 
				if(extract_ext($file)=="pdf"){
					ob_clean();
					download_file_path( $path.$file, $file );
					log\info("download ok '$path$file'");
				}
		}
		*/
?>
