<?php
/*
	Created: 19.01.2022 
	Author: Alex Nosurov
	Description: prepare data to send emails
*/
	require_once(dirname(__FILE__)."/../path.php");
	require_once(proj_path."utils/db_mysql.php");
	require_once(proj_path."view/lang.php");

	function create_user_email($dests, $subj, $msg, $event_id, $user_id, $lang){
		db\execute("insert into emails ".db\compile_insert(["dests" => $dests, "subject" => $subj, "message" => $msg, "lang" => $lang, "event_id" => $event_id, "user_id" => $user_id]) );
		return db\get_last_id();
	}
	function send_user_email($email_id, $settings=null){
		if(!$settings) $settings = get_mail_settings();
		$email = db\select_record("emails","*","id=$email_id");
		$dests = $email["dests"];
		$subj = $email["subj"];
		$message = $email["message"]; // what is inside <body> tag of the message's html
		$lang = $email["lang"];
		$html = lpp_html(replace_keys(file_get_contents(template_path."email"), ["email_message" => $message], 3), $lang);
		try{
			log\debug($html);
			send_mail($sender=$settings["noreply_email"], project_name, $subj, $dests, $html, $settings, $is_html=true, $img_path = proj_path);
			db\execute("update emails set sent=1 where id = $email_id");
			log\info("emails $email_id sent to $dests");
		}catch(Exception $e){
			log\error("Email send fail: ", $e->getMessage());
		}
	}
	// TODO: it just sends single emails
	function send_user_emails($emails){
		$settings = get_mail_settings();
		foreach($emails as $email_id)
			send_user_email($email_id, $settings);
	}

	function email_camp_start($event_id, $camp_id){
		$camp = db\select_record("camps", "id, name camp_name, season camp_season, date_from, date_till, countries", "id=$camp_id");
		$shid = str_replace(";", ",", $camp["countries"]);
		// show list of areas in email
		//$areas = db\select_field_to_array("shops", "distinct area", "id in ($shid)");
		$areas = db\select_field_to_array("locations", "name", "id in ($shid)");
		$a="";foreach($areas as $ar)$a.="<div>$ar</div>";
		$camp["camp_areas"] = $a;
		$subj = '$New_campaign_started$: '.$camp["camp_name"];
		return create_user_emails($event_id, $camp, $subj, "camp_start");
	}
	function render_lbs_list($lbs_list, $lang, $order_id){
		$lbs_html = "";
		$order_path = proj_path_rel_client.get_order_path($order_id);
		foreach($lbs_list as $lbs){
			//$lbs["url"] = base_url.proj_path_rel.files_path.$lbs["path"].$lbs["name_mini"]; works for html page
			//$lbs["url"] = $lbs["path"].get_file_thumb_path($lbs["img_id"], $lbs["name_mini"]); // images are imbedded (is used absolute img_path and each img rel path) ; using FILE_THUMB
			//$lbs["url"] = "table_tree/thumbs/".$lbs["thumb_small"]; // thumb img
			$lbs["url"] = $order_path.$lbs["id"]."/preview.jpg"; // thumb img CROP
			$lbs["w"] = $lbs["r"]-$lbs["l"]+1;
			$lbs["h"] = $lbs["b"]-$lbs["t"]+1;
			//$lbs["delivery_date"] = $order["delivery_date"];
			if(isset($lbs["lb_text"])) $lbs["lb_text"] = join("", array_wrap( preg_split("/\n/", $lbs["lb_text"]), "<div>", "</div>" ) ); // EVITE
			if(isset($lbs["path"])) $lbs["category"] = basename( trim( $lbs["path"], '/' ) );
			$lbs_html .= lpp_html(replace_keys(file_get_contents(template_path."lbs_info"), $lbs, 3), $lang);
		}
		return $lbs_html;
	}

	// send email corresponding to an order to a certain user (with personal links)
	// subj must be in lpp format to be translated
	// msg is what passed from user; normall is same that event.notes (=> TODO: do not need it)
	function create_user_emails($event_id, $data, $subj, $template, $msg=''){
		$settings = get_mail_settings();
		$users = db\select_to_array("v_event_users", "dest_group_id, dest_group_email, order_link, apap_link, triple_link", "event_id = $event_id");
		$emails = array(); // DEF EDEM Excl Double EMails
		$eids = array(); // list of email ids
		foreach($users as $user){
			$email_list = $user["dest_group_email"];
			$email_list = preg_split('/;/', $email_list, -1, PREG_SPLIT_NO_EMPTY); $email_list=array_diff($email_list, $emails); $emails = array_merge($emails, $email_list);  $email_list = join(";", $email_list);  // EDEM
			$lang = lang_get_db( $user["dest_group_id"] , lang_def); // TODO: why not user's own lang?
			$subj = lpp($subj, lang_read($lang));
			$data["url"] = $user["order_link"];
			$data["url_subj"] = $user["apap_link"];
			//$data["message"] = $msg;
			$data["notes"] = $msg;
			if($user["triple_link"])
				$data["durl"] = base_url.proj_path_rel.$user["triple_link"]; // EMAIL_LINK ; individual for each user
			if(isset($data["lbs_list"])) // del order mail not contains lbs list
				$data["lbs_info"] = render_lbs_list($data["lbs_list"], $lang, $order_id=$data["id"]); // lbs info; depends on lang, so personal for each email 
			$html = lpp_html(replace_keys(file_get_contents(template_path.$template), $data, 3), $lang);
			if(!$email_list){
				log\debug("empty email list");
				continue;
			}
			$eids[] = create_user_email($email_list, $subj, $html, $event_id, $user["dest_group_id"], $lang); // TODO: using group id as user id - ok? why?
		}
		return $eids;
	}

	// send email corresponding to an order OR TO LBS if specified
	// msg is what passed from user; normall is same that event.notes
	function create_order_emails($event_id, $order_id, $lbs_id=0, $msg='', $data=null){ // data is used to pass apap link (assoc arr)
		$order = db\select_record("v_orders", "id,shop_id,location,country,camp_name,client_id", "id=$order_id");
		//$shop_id = $order["shop_id"];
		$camp_id = 0;
		$lbs_filter = $lbs_id? "id=$lbs_id" : "order_id=$order_id"; // wholer order or single lbs
		$lbs_list = db\select_to_array("app_lbs", "id, lb_id, path, img_id, name_mini,l,t,r,b,name name_lbs,thumb_small,notes,lb_text",  [$lbs_filter, "img_id<>0"]); // TODO: with CROP no need of img_id, thumb_small
		$order["lbs_list"] = $lbs_list; // lbs list 
		$event = db\select_record("events", "*", "id=$event_id");
		$subj = $event["message"].": ".$order["location"];
		if($data) $order = array_merge($order, $data);
		// moved to send_pending_user_emails if(isset($order["client_id"]))	$order["logo"] = base_url.proj_path_rel."clients/".db\select_field("clients", "code", "id=".$order["client_id"])."/data/bg_logo_medium";
		$order["event_user"] = db\select_field("users", "email", "id=".$event["source_user_id"]);
		return create_user_emails($event_id, $order, $subj, "order_info", $msg);
	}
	//
	function send_orders_email($event_id, $subj, $msg='', $camp_id=0){ // data is used to pass apap link (assoc arr)
		$event = db\select_record("events", "*", "id=$event_id");
		return create_user_emails($event_id, array(), $subj, "orders_info", $msg);
	}


	// BATCH_MAIL

	function send_pending_events($events=null, $check_sessions=true){
		$where = ["sent=0"];
		if($events) $where[] = db\id_in($events, "", "event_id");
		$users = db\select_field_to_array("emails", "distinct user_id", $where); // list of all users for which some not sent email present
		//log\debug($users);
		//$sessions = get_sessions();
		if($check_sessions){
			$sessions = get_dir_contents(proj_path."sessions/"); // SESCO		
			if(sizeof($sessions)) $where[] = "NOT (".db\id_in($sessions, "", "session_id", true).")"; // session control; current sessions
		}
		foreach($users as $user_id){
			$uw = $where; 
			$uw[] = "user_id = $user_id"; // only events for this user
			while($event_id = db\select_field("emails_events", "min(event_id)", $uw)){ // find first not sent email
				$event = db\select_record("events", "*", "id=$event_id");
				$uwe = $uw; // only events of this user and compatible with the first event
				$policy = read_user_settings($user_id, "batch_policy", batch_policy);
				if($policy=="campaign")	$uwe[]= "camp_id=".$event["camp_id"];
				if($policy=="order") 	$uwe[]= "order_id=".$event["order_id"];
				if($policy=="event") 	$uwe[]= "id=".$event["id"];
				// TODO: if($policy=="event_type") $where = "camp_id=".$event["camp_id"]." AND event_type='".$event["type"]."'";
				send_pending_user_emails( db\select_to_array("emails_events", "*", $uwe) );
			}
		}
	}

	function send_pending_user_emails($emails){
		$html = "";
		foreach($emails as $email){
			$email_id = $email["id"];
			$email = db\select_record("emails","*","id=$email_id");
			$html .= "<h1>".$email["subject"]."</h1>".$email["message"];
		}
		$settings = get_mail_settings();
		$dests = $emails[0]["dests"]; // same for all emails
		$lang = $emails[0]["lang"]; // same for all emails
		// TODO: insert client id/code to emails if(isset($order["client_id"]))	$order["logo"] = base_url.proj_path_rel."clients/".db\select_field("clients", "code", "id=".$order["client_id"])."/data/bg_logo_medium";
		$logo = base_url.proj_path_rel."clients/YSL/data/bg_logo_medium";
		$camp = db\select_field("camps", "name", "id=".$emails[0]["camp_id"]);
		$html = lpp_html(replace_keys(file_get_contents(template_path."email"), ["email_message" => $html, "logo" => $logo, "camp_name" => $camp, "tmp" => $dests], 3), $lang);
		try{
			log\debug($html);
			send_mail($sender=$settings["noreply_email"], project_name, project_name.": ".lp("new_events_available"), $dests, $html, $settings, $is_html=true, $img_path = proj_path);
			db\execute("update emails set sent=1 where ".db\id_in($emails));
			log\info("sent ".sizeof($emails)." emails to $dests");
		}catch(Exception $e){
			log\error("Email send fail: ", $e->getMessage());
		}
	}
?>
