<?php
namespace db;
require_once(dirname(__FILE__)."/exception.php");
// TODO: rename to compile_select
function join_where($where){
	if(is_array($where)){
		$where = array_filter($where, function($val){return $val;}); // remove empty conds
		$where = array_map(function($val){return "($val)";}, $where); // wrap into ()
		$where = sizeof($where)? join(" AND ", $where) : "";
	}
	return $where;
}
function build_sql($table, $fields='*', $where='', $order='', $group='', $limit='', $start=0){
	//if($group!='') $fields .= ", $group"; // group field is added here; do not add it to field list  ; commented while could confuse
        $sql = 'SELECT '.$fields.' FROM '.$table;
		$where = join_where($where);
        if($where!='')
                $sql .= ' WHERE '.$where;
        if($group!='')
                $sql .= ' GROUP BY '.$group;
        if($order!='')
                $sql .= ' ORDER BY '.$order;
        if($limit!='')
                $sql .= " LIMIT $start, $limit"; // NOTE: may now work for all dbms
	//\log\debug($sql);
	return $sql;
}

function select($table, $fields='*', $where='', $order='', $group='', $limit='', $start=0){
        execute(  build_sql($table, $fields, $where, $order, $group, $limit, $start));
}

// returns scalar = field value of the first matching record (by idea must be single record)
function select_field($table, $field, $where='', $order=''){
	global $response;
	global $record;
	select($table, $field, $where, $order);
	if( ! get_count() ) return null;
	fetch(); 
	// next supposes that a records is an assoc array
	// NOTE: do not use get_field($field) while $field may be an expr like count(*)
	reset($record);
	return current($record);
}
function select_sum($table, $field, $where){ // just to avoid checking res null
	$res = select_field($table, "sum($field)", $where);
	return ($res? $res : 0);
}
// returns first matching record
// should return empty of no rec exists
function select_record($table, $fields, $where='', $order=''){
	global $response;
	global $record;
	select($table, $fields, $where, $order);
	return fetch(); 
}
// returns the field of current record (need fetch before)
function get_field($field){
	global $record;
	if( !array_key_exists($field, $record) )
		throw new \db_exception("field not exists: $field");
	return $record[$field];
}
// returns all fields of current record (need fetch before)
function get_all_fields(){
	global $record;
	return $record;
}

// input: array of params: {field, value, operation, string, predicative (used for crit=array, def='or')}
// operation = [=,<,>,like]
// string = 0/1 indicates if wrap into quotes; 
// note: maybe need inform if need escaping;
function compile_where($cond_array, $cond_pred="AND"){
	$preds = array();
	foreach($cond_array as $cond){	
		$apice = $cond['string']? "'": "";
		$percent = $cond['operation']=='like'? '%' : '';
		$value = $cond['value'];
		if( is_array( $value ) ){ // vector
			$pr = array();
			foreach($value as $val){
				if($cond['string'])
					$val = escape($val);
				$pr[] = $cond['field'].' '.$cond['operation'].' '.$apice.$percent.$val.$percent.$apice;
			}
			$pred = get_or_def($cond, "predicative", "OR");
			$preds[] = '('.join(" $pred ", $pr).')';
		}
		else{ // scalar
			if($cond['string'])
				$value = escape($value);
			$preds[] = $cond['field'].' '.$cond['operation'].' '.$apice.$percent.$value.$percent.$apice;
		}
	}
	return join(" $cond_pred ", $preds);
}
// table - is array of field definitions: elem: name => array()
function compile_check($values, $table){
	$upd = array();
	if(!$table) $table = array();
	foreach($values as $fname => $val){
		$len = get_or_def2($table, $fname, "length", 0);
		if( $len && strlen($val)> $len )
			if( get_or_def2($table, $fname, "can_truncate", 0) )
				$val = substr($val, 0, $len);
			else
				throw new db_exception("value too long for column $fname/$len: '$val'");
		$type = get_or_def2($table, $fname, "type", "string");
		if( $type=="string" && get_or_def2($table, $fname, "escape", 1) && $val!="0" /*escape turns '0' to ''*/ )
			$val = escape($val);
		// do it anyway if( $type=="string" ) //
			$val = "\"$val\"";
		if( $type=="number" && !$val )
			$val = 0;
		$upd[$fname] = $val;
	}
	return $upd;
}
// returns: (fn,fn...) values (val, val...) ; so need prepend: INSERT INTO <table name>
function compile_insert($values, $table=null){
	$upd = compile_check($values, $table);
	if(sizeof($upd)==0)return null;
	return "(".join(", ", array_keys($upd)).") VALUES (".join(", ", array_values($upd)).") ";
}
// returns fn=val, fn=val...
function compile_update($values, $table=null){
	$upd = compile_check($values, $table);
	if(sizeof($upd)==0)return null;
	//\log\info($upd);
	$res = array(); foreach($upd as $fname => $val) $res []= "$fname = $val";
	return join(", ", $res)." ";
}
/* version which iterates array of defs
function compile_update($values, $table){
	$upd = array();
	foreach($table as $fname => $fdef){
		if( isset($values[$fname]) ){
			$val = $values[$fname];
			$len = get_or_def($fdef, "length", 0);
			if( $len && sizeof($val)> $len )
				if( get_or_def($fdef, "can_truncate", 0) )
					$val = substr($val, 0, $len);
				else
					throw new db_exception("value too long for column $fname/$len: '$val'");
			$type = get_or_def($fdef, "type", "string");
			if( $type=="string" && get_or_def($fdef, "escape", 1) )
				$val = escape($val);
			if( $type=="string" )
				$val = "\"$val\"";
			$upd[]= "$fname = $val";
		}
	}
	if(sizeof($upd)==0)return null;
	return join(", ", $upd);
}
*/
// returns array recs; rec = ass array field->val (if field='') else -  scalar: field value
// NEW 11/11/19 added memory clear
function response_to_array($field=''){ // TODO: maybe optimize a bit
	$result_array = array();
	while($rec = fetch())
		if($field=='') // all fields
			$result_array[] = $rec;
		else
			$result_array[] = $rec[$field];
	// TDOO: unset response
	free();
	return $result_array;
}
function select_to_array($table, $field="*", $where='', $order='', $group='', $limit=''){
	select($table, $field, $where, $order, $group, $limit);
	return response_to_array();
}

function select_field_to_array($table, $field, $where='', $order='', $group='', $limit=''){
	select($table, $field, $where, $order, $group, $limit);
	$res = response_to_array();
	if(sizeof($res)) $field = array_keys($res[0])[0]; // useful when field is sms complex, eg "distinct field"
	return array_column($res, $field); // TODO: may be not unique
}
// example: record is (id, name); it transforms ("id"=>id, "name"=>name) to assoc arr (id=>name)
function select_id_field_to_array($table, $id_field, $value_field, $where='', $order='', $group='', $limit=''){
	select($table, "$id_field, $value_field", $where, $order, $group, $limit);
	$res = array();
	while($rec = fetch())
		$res[ $rec[$id_field] ] = $rec[ $value_field ];
	return $res;
}


function now(){
	$dt = new \DateTime();
	return $dt->format(get_date_format());
}
// in = DateTime; out = date time in db format; $diff - minutes to add
function date_add($diff, $from=''){
	if($from=='')
		$from = new \DateTime('now');
	if($diff>0)
		$from->add( new \DateInterval("PT".$diff."M") );
	else
		$from->sub( new \DateInterval("PT".(-$diff)."M") );
	return $from->format(get_date_format());
}
// builds cond like where id in( IDs from $arr )
function id_in($arr, $id="id", $field="id", $is_str=false){ // $id - field name in array; $field - field name in db (in condition issued)
	if(!$arr || !sizeof($arr)) return ""; // EIH
	$r = array();
	foreach($arr as $e) {
		if( is_array($e) ) $e = $e[$id]; // assoc arr with elem id=>   or id itself
		if($is_str) $e = "'$e'";
		$r[]= $e; // assoc arr with elem id=>   or id itself
	}
	if(!sizeof($r)) $r = array(-1); // NOTE: quickfix to amend situation when id list empty  TODO: this not works for EIH
	return "$field in (".join(",", $r).")";
}
// used when need to create cond which checks if a table filed which represents concatenated list of vals (eg val1;val2;val3)
function in_list($field, $val, $sep=';'){ return "($field='$val' or $field like '$val$sep%' or $field like '%$sep$val$sep%' or $field like '%$sep$val')"; }

// updates several records at once
/*function update_records($table, $recs, $id_field, $data_fields_str, $data_fields_int){
	foreach($recs as $rec){
		$upd = array();
		foreach($data_fields_int as $field)
			$upd[] = $field = $rec[$field];
		foreach($data_fields_str as $field)
			$upd[] = $field = "'".$rec[$field]."'";
		execute("update $table */
?>
