<?php
namespace db;
require_once(dirname(__FILE__)."/exception.php");
require_once(dirname(__FILE__)."/../path.php");  // for PORTAL
require_once(dirname(__FILE__)."/db.php");
$connection = false;
$response = false;
$record = false;
function concat($a,$b){ return "concat($a,$b)"; } // used this to perform concatenation (to make queries dbms independent)
function filename($field){ return "SUBSTRING_INDEX($field, '/', -1)"; } // used this to perform concatenation (to make queries dbms independent)


//  NOTE     functions are called from another file, which not has global variables declared here; thus they must be declared with global in each function (think that these functions are in file-caller)

//----------------------------------------------------------------------------------------------
//function connect($settings){ must be run on every inclusion, so out of function
//$settings = parse_ini_file(dirname(__FILE__).'/../settings.ini', true);
$settings = get_settings(); // PORTAL
$dbs = $settings['db']; 
$connection = mysqli_connect($dbs['host'], $dbs['user'], $dbs['password'], $dbs['dbname']);
if(mysqli_connect_errno()){
	$connection = false;
	throw new \db_exception("Failed to connect database: " + array_to_str($dbs) + "; error: ".mysqli_connect_error());
}
//----------------------------------------------------------------------------------------------
function connected(){global $connection; return !empty($connection);}   // checks if connected; now used in alert handler to manage the situation when alert was caused by db fail (and can't insert alert to db)
function get_db_settings(){global $dbs; return $dbs;} // used in TABREP only

function execute($sql){ 
	global $connection;
	global $response;
	\log\write("debug2", $sql);
	$response = mysqli_query($connection, $sql);
	if( $response===false)
		throw new \db_exception(mysqli_error($connection));
	return $response;
}	

function free(){
	$response = false;
}
// not saves result to 'response' but just returns it; used to perform another query when 'response' is already busy
function execute_to($sql){
	global $connection;
        //
	\log\write("debug2", $sql);
        $response_local = mysqli_query($connection, $sql);
	if( $response_local === false)
		throw new \db_exception(mysqli_error($connection));
	return $response_local;
}

function get_count(){
	global $response;
	return $response==false? 0: mysqli_num_rows($response);
}

function get_count_of($response){
	return $response==false? 0: mysqli_num_rows($response);
}

function fetch(){
	global $response;
	global $record;
	return $record = mysqli_fetch_assoc($response);
}

function fetch_from(&$response){
	return mysqli_fetch_assoc($response);
}
function escape(&$str){
	global $connection;
	if(!$str) return "";
	return mysqli_real_escape_string($connection, $str);
}
// not ref
function escape2($str){
	global $connection;
	return mysqli_real_escape_string($connection, $str);
}

// returns database specific date format (in PHP codings, to be used by date() function); used instead of date_to_str
function get_date_format(){
	return "Y-m-d H:i:s";
}
// this function depends actually on project used, so should be included in separate file (like db_ic.php)
function make_path(){
	//return "CONCAT( files.full_path_directory, substr(path, 2, length(path)-1))"; // full_path_directory + path without the leading slash (path contains at least file name)
	return 'percorso';
}

function get_last_id(){
	global $connection;
 	return mysqli_insert_id($connection);
}
// db: a table containing list of paths which represent some folder tree
// input: a folder name (inside tree represented in db); path field name
// output: list of child folders or the parent folder (is stored in db response, field name = 'child')
// usage: used when navigating a dir tree represented by a table
function open_dir_db($parentDir, $table, $path_field, $cond=''){
	$pd_len = strlen($parentDir); 
	if(!is_array($cond)) $cond = [$cond];
	// next query contains group operator => can't select other fields here (represents not records but group result)
	//$sql = "SELECT distinct substring_index( substr(F.P, $pd_len+1, length(F.P)-$pd_len), '/', 1) AS child FROM (SELECT $path_field AS P FROM $table where $path_field like '".escape($parentDir)."%' $and_cond) AS F order by child";
	$sql = "SELECT distinct substring_index( substr(F.P, $pd_len+1, length(F.P)-$pd_len), '/', 1) AS child FROM (SELECT $path_field AS P FROM $table where ".join_where(array_add($cond, "$path_field like '".escape($parentDir)."%'")).") AS F order by child";
	execute($sql);
}
// returns (files=>arr, folders=>arr); a folder = path segment, no end-slash
// the main problem with open_dir_db is that it returns mixed array of files/folder and the client needs check each record (make rec_count queries) to find out which are files (and get other necessary fields)
// this version returns two lists of paths/files but needs presence of "name" field; need table/view with fields path/name (path = full path, includes name)
// table_simple: in MS files-table is slow view; so to select sub-folders it's better to use the table
function open_dir_db1($parent_dir, $table, $table_simple, $file_fields, $cond='', $order='', $what='files;folders'){
	if(!is_array($cond)) $cond = [$cond];
	$files=array();$folders=array();
	// files
	if( strpos($what, "files")!==false)
		$files = select_to_array($table, $file_fields, array_add($cond, "path = '".escape($parent_dir)."'"), $order);  // PATH/NAME, path without file name
	$pd_len = strlen($parent_dir); 
	// next query contains group operator => can't select other fields here (represents not records but group result)
	// TODO: FASTER can make it faster if use field files.parent_path (=pathname(path))
	$sql = "SELECT distinct substring_index( substr(F.P, $pd_len+1, length(F.P)-$pd_len), '/', 1) AS child FROM (SELECT path AS P FROM $table_simple where ".join_where(array_add($cond, "path like '".escape($parent_dir)."_%'")).") AS F order by child"; // cut parent_dir and take first elem; need '_' to avoid empty elems (same dir)
	if( strpos($what, "folders")!==false){
		execute($sql);
		$folders = response_to_array();
	}
	return array("files" => $files, "folders" => $folders);
}

function exists($table, $where){
	$response = execute_to("select exists (select 1 from $table where $where)"); 
	$record = mysqli_fetch_assoc($response);
	reset($record);
	return current($record);
}	

// allows to insert a rec with one blob value ; TODO: donno if has sense, while still using 's' as type
// $sql contains insert statement with on '?' representing blob, eg 'insert into files(contents) values(?)'
function execute_blob($sql, $blob){
	global $connection;
	\log\debug($sql);
	$st = mysqli_prepare($connection, $sql); // TODO: check errs
	if(!$st)
		throw new \Exception("prepare statement fail: $sql");
	mysqli_stmt_bind_param($st, 's', $blob);
	$blob = mysqli_real_escape_string($connection, $blob);
	$res = mysqli_stmt_execute($st);
	mysqli_stmt_close($st);
	if(!$res)
		throw new \Exception("execute_blob error: ".mysqli_error($connection)); // TODO: not shows error
}

	// DEF TABREP table repair; current scheme: controller/while - calls check_tables with repair=true; thus an alarm is state page quite useless (only for the case when repair fails)
	function check_tables($repair=false){
		\log\info("Starting db consistency check; repair=$repair");
		$sett = get_db_settings();
		$db_name = $sett["dbname"];
		$tables = select_to_array("INFORMATION_SCHEMA.TABLES", "TABLE_NAME", "TABLE_SCHEMA='$db_name' AND TABLE_TYPE='BASE TABLE'");
		$res_all = "";
		foreach($tables as $table){
			$table = $table["TABLE_NAME"];
			\log\info("Checking table $table...");
			$res = shell_exec("mysqlcheck --silent $db_name $table");
			if($res){ // if error
				\log\error("Table '$table' is corrupt: $res");
				if($repair){
					\log\info("Backupping $table...");
					shell_exec("mysqldump -u {$sett['user']} -p {$sett['password']}  $db_name $table > ".backup_path."$table.sql");
					\log\info("Correcting...");
					$res = shell_exec("mysqlcheck --repair $db_name $table");
					\log\info($res);
					//\log\info("Checking table $table...");
					$res = shell_exec("mysqlcheck --silent $db_name $table");
					if($res)
						\log\error("Failed to repair table: $res");
					else
						\log\info("Table repaired");
				}
				$res_all.="\n$res"; // NOTE: thus $res always shows last check res (with or without repair)
			}
		}
		\log\info("End of db consistency check");
		return $res_all; // returns not repaired tables
	}

	function replace_wrong_chars($str){
		$regex = <<<'END'
/
  (
    (?: [\x00-\x7F]                 # single-byte sequences   0xxxxxxx
    |   [\xC0-\xDF][\x80-\xBF]      # double-byte sequences   110xxxxx 10xxxxxx
    |   [\xE0-\xEF][\x80-\xBF]{2}   # triple-byte sequences   1110xxxx 10xxxxxx * 2
    |   [\xF0-\xF7][\x80-\xBF]{3}   # quadruple-byte sequence 11110xxx 10xxxxxx * 3 
    ){1,100}                        # ...one or more times
  )
| .                                 # anything else
/x
END;
		return preg_replace($regex, "$1", $str);
	}

	// used to rename a path - replace some inner segment of all matching paths
	function move_dir($table, $field, $path_old, $path_new, $check_exist=true){
		$name = basename(trim($path_old, '/'));
		$dir_old = dirname(trim($path_old, '/')); // = dir_new
		$dir_old = $dir_old=='.'? '' : $dir_old.'/';
		$dol = strlen($dir_old) + 1;
		$name = escape($name);
		$path_new = escape($path_new);
		$dir_old = escape($dir_old);
		if($check_exist &&  exists($table, "$field = '$path_new$name'") )
			return false;
		execute("update $table set $field = concat('$path_new', substring($field, $dol) ) where $field like '$path_old%'");
		return true;
	}

	// used to rename a path - replace some inner segment of all matching paths
	function rename_dir($table, $field, $path_old, $name_new, $check_exist=true){
		$name_old = basename(trim($path_old, '/'));
		$dir_old = dirname(trim($path_old, '/')); // = dir_new
		$dir_old = $dir_old=='.'? '' : $dir_old.'/';
		$name_new = escape($name_new).'/' ;
		$path_new = escape2($dir_old.$name_new);
		$path_old = escape($path_old);
		$dir_old = escape($dir_old);
		if($check_exist &&  exists($table, "$field = '$path_new'") )
			return false;
		execute("update $table set $field = concat('$dir_old', '$name_new', substring($field, length('$path_old')+1) ) where $field like '$path_old%'");
		return true;
	}

	function is_db_dos_error($str){
		return strpos($str, "MySQL server has gone away")!==false; // TODO: need include other cases
	}
?>
